/*
 *  Copyright (C) 1998  Luca Deri <deri@unipi.it>
 *                      Stefano Suin <stefano@unipi.it>
 *                      
 *  					Centro SERRA, University of Pisa
 *  					http://www-serra.unipi.it/
 *  					
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef linux
/* 
   This allows to hide the (minimal) differences between
   linux and BSD 
*/
#ifndef _BSD_SOURCE
#define _BSD_SOURCE
#endif
#endif

#include <errno.h>

#ifdef HAVE_NCURSES_H
#include <ncurses.h>
#else
#ifdef HAVE_CURSES_H
#include <curses.h>
#endif
#endif

#ifndef A_NORMAL
#define A_NORMAL        000000000000L
#endif

#ifndef A_REVERSE
#define A_REVERSE       000040000000L
#endif

#include <ctype.h>
#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/fcntl.h>


#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif

#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#include <net/if.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <netinet/in.h>

#ifdef HAVE_NETINET_IF_ETHER_H
#include <netinet/if_ether.h>
#endif

#ifdef HAVE_NETINET_IN_SYSTM_H
#include <netinet/in_systm.h>
#endif

#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netdb.h> 
#include <stdio.h>
#include <net/bpf.h>
#include <pcap.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ethertype.h>
#include <setjmp.h>

#include "interface.h"
#include "setsignal.h"

#define ETHERNET_ADDRESS_LEN 6

/* ******************************************* */

/*
 * The definitions below have been copied
 * from llc.h that's part of tcpdump
 *
 */

struct llc {
  u_char dsap;
  u_char ssap;
  union {
    u_char u_ctl;
    u_short is_ctl;
    struct {
      u_char snap_ui;
      u_char snap_pi[5];
    } snap;
    struct {
      u_char snap_ui;
      u_char snap_orgcode[3];
      u_char snap_ethertype[2];
    } snap_ether;
  } ctl;
};

#define	llcui		ctl.snap.snap_ui
#define	llcpi		ctl.snap.snap_pi
#define	orgcode		ctl.snap_ether.snap_orgcode
#define	ethertype	ctl.snap_ether.snap_ethertype
#define	llcis		ctl.is_ctl
#define	llcu		ctl.u_ctl

#define	LLC_U_FMT	3
#define	LLC_GSAP	1
#define LLC_S_FMT	1

#define	LLC_U_POLL	0x10
#define	LLC_IS_POLL	0x0001
#define	LLC_XID_FI	0x81

#define	LLC_U_CMD(u)	((u) & 0xef)
#define	LLC_UI		0x03
#define	LLC_UA		0x63
#define	LLC_DISC	0x43
#define	LLC_DM		0x0f
#define	LLC_SABME	0x6f
#define	LLC_TEST	0xe3
#define	LLC_XID		0xaf
#define	LLC_FRMR	0x87

#define	LLC_S_CMD(is)	(((is) >> 10) & 0x03)
#define	LLC_RR		0x0100
#define	LLC_RNR		0x0500
#define	LLC_REJ		0x0900

#define LLC_IS_NR(is)	(((is) >> 1) & 0x7f)
#define LLC_I_NS(is)	(((is) >> 9) & 0x7f)

#ifndef LLCSAP_NULL
#define	LLCSAP_NULL		0x00
#endif

#ifndef LLCSAP_GLOBAL
#define	LLCSAP_GLOBAL		0xff
#endif

#ifndef LLCSAP_8021B
#define	LLCSAP_8021B_I		0x02
#endif

#ifndef LLCSAP_8021B
#define	LLCSAP_8021B_G		0x03
#endif

#ifndef LLCSAP_IP
#define	LLCSAP_IP		0x06
#endif

#ifndef LLCSAP_PROWAYNM
#define	LLCSAP_PROWAYNM		0x0e
#endif

#ifndef LLCSAP_8021D
#define	LLCSAP_8021D		0x42
#endif

#ifndef LLCSAP_RS511
#define	LLCSAP_RS511		0x4e
#endif

#ifndef LLCSAP_ISO8208
#define	LLCSAP_ISO8208		0x7e
#endif

#ifndef LLCSAP_PROWAY
#define	LLCSAP_PROWAY		0x8e
#endif

#ifndef LLCSAP_SNAP
#define	LLCSAP_SNAP		0xaa
#endif

#ifndef LLCSAP_ISONS
#define	LLCSAP_ISONS		0xfe
#endif

#ifndef LLCSAP_NETBIOS
#define	LLCSAP_NETBIOS		0xf0
#endif

/* ******************************* */

#ifndef min
#define min(a,b) ((a)>(b)?(b):(a))
#endif

/* SunOS 4.x, at least, doesn't have a CLOCKS_PER_SEC.  I got this value
 * from Solaris--who knows.             
 * Paul D. Smith <psmith@baynetworks.com>
 */
#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC 1000000
#endif

#ifndef NTOHL
#define NTOHL(x)    (x) = ntohl(x) 
#endif

#define HASHNAMESIZE 4096

/* Stefano Suin <stefano@unipi.it> */
#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN  64
#endif

/*
 * Max number of screen switchable using
 * the space bar (interactive mode only).
*/
#define MAX_NUM_SCREENS  6

/*
 * Max number of TCP sessions that can be 
 * recorded per host 
 */
#define MAX_NUM_TCP_SESSIONS  32

#define SERVICE_HASH_SIZE 512

#define NULL_HDRLEN 4

#define SHORT_REPORT        1
#define LONG_REPORT         2


#define MAX_NUM_SESSION_PEERS 5

#define CLIENT_TO_SERVER       1
#define CLIENT_FROM_SERVER     2
#define SERVER_TO_CLIENT       3
#define SERVER_FROM_CLIENT     4


#define CLIENT_ROLE            1
#define SERVER_ROLE            2

#define LOCAL_HOST             1
#define REMOTE_HOST            2


#define REMOTE_TO_LOCAL_ACCOUNTING   1
#define LOCAL_TO_REMOTE_ACCOUNTING   2
#define LOCAL_TO_LOCAL_ACCOUNTING    3

/* ************* Types Definition ********************* */

struct hnamemem {
  struct in_addr addr;
  char *name;
};

struct enamemem {
  u_short e_addr0;
  u_short e_addr1;
  u_short e_addr2;
  char   *e_name;
  u_char *e_nsap;  /* used only for nsaptable[] */
  struct enamemem *e_nxt;
};

typedef struct trafficEntry {
  unsigned long bytesSent, bytesReceived;
} TrafficEntry;

typedef struct serviceEntry {
  u_short port;
  char* name;
} ServiceEntry;

struct hostTraffic;
/* IP Session global information */

typedef struct ipGlobalSession {
  u_short port;                    /* port (either client or server)           */
  u_char initiator;                /* CLIENT_ROLE/SERVER_ROLE                  */
  time_t firstSeen;                /* time when the session has been initiated */
  time_t lastSeen;                 /* time when the session has been closed    */
  u_short sessionCounter;          /* # of sessions we've observed             */
  unsigned long bytesSent;         /* # bytes received (peer -> initiator)     */
  unsigned long bytesReceived;     /* # bytes received (peer -> initiator)     */
  u_char lastPeer;                 /* idx of the last peer added to the list   */
  struct hostTraffic *peers[MAX_NUM_SESSION_PEERS]; /* session peers           */
  struct ipGlobalSession  *next;   /* next element (linked list)               */
} IpGlobalSession;

#define MAX_NUM_FIN          4


/* IP Session Information */
typedef struct ipSession {
  struct hostTraffic *initiator;   /* initiator address   (IP address)         */
  u_short sport;                   /* initiator address   (port)               */
  struct hostTraffic *remotePeer;  /* remote peer address (IP address)         */
  u_short dport;                   /* remote peer address (port)               */
  time_t firstSeen;                /* time when the session has been initiated */
  time_t lastSeen;                 /* time when the session has been closed    */
  unsigned long bytesSent;         /* # bytes sent (initiator -> peer)         */
  unsigned long bytesReceived;     /* # bytes received (peer -> initiator)     */
  u_short  numFin;                 /* # FIN pkts received                      */
  u_short  numFinAcked;            /* # ACK pkts received                      */
  tcp_seq finId[MAX_NUM_FIN];      /* ACK ids we're waiting for                */
  unsigned long lastFlags;         /* flags of the last TCP packet             */
  u_int32_t lastCSAck, lastSCAck;  /* they store the last ACK ids C->S/S->C    */
  u_int32_t lastCSFin, lastSCFin;  /* they store the last FIN ids C->S/S->C    */
  u_short sessionState;            /* actual session state                     */
} IPSession;


/* Host Traffic */
typedef struct hostTraffic {
  struct in_addr hostIpAddress;
  u_char         ethAddress[ETHERNET_ADDRESS_LEN];
  char           ethAddressString[18];
  char           hostNumIpAddress[64];
  u_char         subnetLocalHost; /* indicates whether the host is either LOCAL_HOST or REMOTE_HOST */
  unsigned long  pktSent, pktReceived, pktBroadcastSent, pktMulticastSent;
  unsigned long  lastBytesSent, bytesSent, bytesSentLocally, bytesSentRemotely;
  unsigned long  lastBytesReceived, bytesReceived, bytesReceivedLocally, bytesReceivedFromRemote;
  /* IP */
  unsigned long  tcpSentLocally, tcpSentRemotely, udpSentLocally, udpSentRemotely, icmpSent;
  unsigned long  tcpReceivedLocally, tcpReceivedFromRemote, udpReceivedLocally, udpReceivedFromRemote, icmpReceived;
  unsigned long  ftpSentLocally, ftpSentRemotely, ftpReceivedLocally, ftpReceivedFromRemote;
  unsigned long  httpSentLocally, httpSentRemotely, httpReceivedLocally, httpReceivedFromRemote;
  unsigned long  dnsSentLocally, dnsSentRemotely, dnsReceivedLocally, dnsReceivedFromRemote;
  unsigned long  telnetSentLocally, telnetSentRemotely, telnetReceivedLocally, telnetReceivedFromRemote;
  unsigned long  netbios_over_ip_SentLocally, netbios_over_ip_SentRemotely, 
    netbios_over_ip_ReceivedLocally, netbios_over_ip_ReceivedFromRemote;
  unsigned long  popSentLocally, popSentRemotely, popReceivedLocally, popReceivedFromRemote;
  unsigned long  snmpSentLocally, snmpSentRemotely, snmpReceivedLocally, snmpReceivedFromRemote;
  unsigned long  nfsSentLocally, nfsSentRemotely, nfsReceivedLocally, nfsReceivedFromRemote;
  unsigned long  x11SentLocally, x11SentRemotely, x11ReceivedLocally, x11ReceivedFromRemote; /* X11 */
  /* non IP */
  unsigned long  ipxSent, ipxReceived;
  unsigned long  dlcSent, dlcReceived;
  unsigned long  arp_rarpSent, arp_rarpReceived;
  unsigned long  decnetSent, decnetReceived;
  unsigned long  appletalkSent, appletalkReceived;
  unsigned long  netbiosSent, netbiosReceived;
  unsigned long  otherSent, otherReceived;
  /* list of sessions initiated/received by this host */
  IpGlobalSession *tcpSessionList, *udpSessionList; 
} HostTraffic;

/* **************************** */

/* Packet buffer */
struct pbuf {
  struct pcap_pkthdr h;
  u_char b[sizeof(unsigned int)];	/* actual size depend on snaplen */
};

/* **************************** */

#ifdef ETHER_HEADER_HAS_EA
#define ESRC(ep) ((ep)->ether_shost.ether_addr_octet)
#define EDST(ep) ((ep)->ether_dhost.ether_addr_octet)
#else
#define ESRC(ep) ((ep)->ether_shost)
#define EDST(ep) ((ep)->ether_dhost)
#endif

/* **************************** */

#define MULTICAST_MASK 0xe00000 /* 224.X.Y.Z */

/* **************************** */

#define LONG_FORMAT    1
#define SHORT_FORMAT   2

/* **************************** */

/* TCP Session State Transition */

#define STATE_BEGIN             0
#define STATE_FIN1_ACK0         1
#define STATE_FIN1_ACK1         2
#define STATE_FIN2_ACK0         3
#define STATE_FIN2_ACK1         4
#define STATE_FIN2_ACK2         5
#define STATE_TIMEOUT           6
#define STATE_END               7

/* **************************** */

/* Timedout sessions are scanned each SESSION_SCAN_DELAY seconds */
#define SESSION_SCAN_DELAY        30  /* 30 secs   */

/* This is the 2MSL timeout as defined in the TCP standard (RFC 761) */
#define TWO_MSL_TIMEOUT          120  /* 2 minutes */
#define DOUBLE_TWO_MSL_TIMEOUT   (2*TWO_MSL_TIMEOUT)

/* **************************** */

#define COLOR_1               "#CCCCFF"
#define COLOR_2               "#FFCCCC"
#define COLOR_3               "#EEEECC"

#define BUF_SIZE              768

/* **************************** */

#ifndef WNOHANG
#define WNOHANG 1
#endif
